@@ -56,7 +56,7 @@ args = parser.parse_args()
 trunc = args.trunc_print_len
 
 max_num_seqs = args.max_num_seqs  # defines the max batch size
-assert args.max_prompt_len < args.max_model_len
+assert args.max_prompt_len <= args.max_model_len
 
 if platform.machine() == "arm64":
     print("Detected arm64 running environment. "
@@ -68,7 +68,6 @@ if platform.machine() == "arm64":
 if "VLLM_SPYRE_DYNAMO_BACKEND" not in os.environ:
     os.environ['VLLM_SPYRE_DYNAMO_BACKEND'] = 'eager'
 os.environ['VLLM_SPYRE_USE_CB'] = '1'
-os.environ['VLLM_USE_V1'] = '1'
 
 template = ("Summarize the following code: \n\n{}")
 
@@ -122,7 +121,7 @@ def round_up(t):
 
 
 tokens_to_generate = [
-    args.max_model_len - round_up(plen) for plen in prompt_lens
+    args.max_model_len + 1 - round_up(prompt_len) for prompt_len in prompt_lens
 ]
 
 sampling_params = [
