@@ -1,17 +1,6 @@
-# 🌶️🌶️🌶️ Hack to allow testing of both engines
+import hashlib
 import os
-
-# If `VLLM_USE_V1=1` is set upon first vLLM import, then there is a side effect
-# that will cause the V1 engine to always be selected. This is intentionally
-# done for backwards-compatibility of code that was using the AsyncLLMEngine
-# constructor directly, instead of using the `.from_engine_args` construction
-# methods that will select the appropriate v0 or v1 engine. See:
-# https://github.com/vllm-project/vllm/blob/v0.8.4/vllm/engine/llm_engine.py#L2169-L2171
-# Deleting VLLM_USE_V1 here before importing vLLM allows us to continue testing
-# both engines.
-if "VLLM_USE_V1" in os.environ:
-    del os.environ["VLLM_USE_V1"]
-# 🌶️🌶️🌶️ end hack
+import random
 
 import pytest
 import torch
@@ -29,7 +18,7 @@ os.environ["VLLM_WORKER_MULTIPROC_METHOD"] = "spawn"
 def pytest_collection_modifyitems(config, items):
     """ Mark all tests in e2e directory"""
     for item in items:
-        if "tests/e2e" in str(item.nodeid):
+        if "e2e" in str(item.nodeid):
             item.add_marker(pytest.mark.e2e)
 
 
@@ -95,8 +84,7 @@ def remote_openai_server(request):
         max_num_seqs = params["max_num_seqs"]
         env_dict = {
             "VLLM_SPYRE_USE_CB": "1",
-            "VLLM_SPYRE_DYNAMO_BACKEND": backend,
-            "VLLM_USE_V1": "1"
+            "VLLM_SPYRE_DYNAMO_BACKEND": backend
         }
         server_args = [
             "--max_num_seqs",
@@ -118,8 +106,6 @@ def remote_openai_server(request):
             ','.join(map(str, warmup_batch_size)),
             "VLLM_SPYRE_DYNAMO_BACKEND":
             backend,
-            "VLLM_USE_V1":
-            "1"
         }
 
         # Default to None if not present
@@ -139,3 +125,11 @@ def remote_openai_server(request):
             yield server
     except Exception as e:
         pytest.fail(f"Failed to setup server: {e}")
+
+
+@pytest.fixture
+def set_random_seed(request):
+    func_hash = hashlib.sha256(request.node.originalname.encode('utf-8'))
+    seed = int(func_hash.hexdigest(), 16)
+    random.seed(seed)
+    yield
