@@ -8,14 +8,16 @@ Run `python -m pytest tests/e2e/test_spyre_cb_inference_steps.py`.
 
 import pytest
 from scheduling_utils import check_scheduler_inference_steps
-from spyre_util import get_spyre_backend_list, get_spyre_model_list
+from spyre_util import (check_output_against_hf, get_spyre_backend_list,
+                        get_spyre_model_list)
 
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_prompts_aligned_with_tkv_boundaries(model: str, backend: str,
-                                             monkeypatch: pytest.MonkeyPatch):
+                                             monkeypatch: pytest.MonkeyPatch,
+                                             set_random_seed: None):
     """ Scenario where it happens that all the sequences get scheduled in a 
     fashion where they are aligned with the block boundaries (i.e. tkv multiple 
     of 64 at the time of prefilling).
@@ -33,6 +35,7 @@ def test_prompts_aligned_with_tkv_boundaries(model: str, backend: str,
     steps_add_reqs = [0, 0, 0]  # add all requests in the beginning
     available_blocks = -1  # no restriction
     max_num_seqs = 2
+    max_model_len = 256
 
     checked_steps = [
         {
@@ -92,26 +95,26 @@ def test_prompts_aligned_with_tkv_boundaries(model: str, backend: str,
         },
         {
             # Prefill sequence 2
-            # total blocks in use: 4 - 2 + 2 = 4
+            # total blocks in use: 4 - 2 + 1 = 3
             "step": 67,
             "tkv": 128,  # Tkv doesn't increase because it is a prefill
             "waiting": [],
             "running": ["2", "1"],
             "request_outputs": ["2"],
-            # 5 - 2 (seq 0) + 3 (prefill (2 blocks) + decodes (1 block))
-            "n_reserved_blocks": 6,
-            "n_used_blocks": 4
+            # 5 - 2 (seq 0) + 2 (prefill (1 block) + decodes (1 block))
+            "n_reserved_blocks": 5,
+            "n_used_blocks": 3
         },
         {
             # Decode sequences 1 and 2
-            # total blocks in use: 4 + 2 = 6
+            # total blocks in use: 3 + 2 = 5
             "step": 68,
             "tkv": 129,
             "waiting": [],
             "running": ["2", "1"],
             "request_outputs": ["2", "1"],
-            "n_reserved_blocks": 6,
-            "n_used_blocks": 6
+            "n_reserved_blocks": 5,
+            "n_used_blocks": 5
         },
         {
             # Sequence 1 finishes at step 69
@@ -122,18 +125,18 @@ def test_prompts_aligned_with_tkv_boundaries(model: str, backend: str,
             "running": ["2"],
             "request_outputs": ["2", "1"],
             "finished_requests": ["1"],
-            "n_reserved_blocks": 6,
-            "n_used_blocks": 6
+            "n_reserved_blocks": 5,
+            "n_used_blocks": 5
         },
         {
             # Decode sequence 2
-            # total blocks in use: 6 - 3 - 1 (remove padded block) = 2
+            # total blocks in use: 5 - 3  = 2
             "step": 70,
             "tkv": 67,  # tkv is reset by 64 due to removing the padded block
             "waiting": [],
             "running": ["2"],
             "request_outputs": ["2"],
-            # 6 - 3 (seq 1 left) - 1 (removing the padded block)
+            # 5 - 3 (seq 1 left)
             "n_reserved_blocks": 2,
             "n_used_blocks": 2
         },
@@ -161,7 +164,7 @@ def test_prompts_aligned_with_tkv_boundaries(model: str, backend: str,
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -170,16 +173,21 @@ def test_prompts_aligned_with_tkv_boundaries(model: str, backend: str,
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
 
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_prompts_misaligned_with_tkv_boundaries(
-        model: str, backend: str, monkeypatch: pytest.MonkeyPatch):
+        model: str, backend: str, monkeypatch: pytest.MonkeyPatch,
+        set_random_seed: None):
     """ Scenario where it happens that some sequence gets scheduled in a way 
     that it is misaligned with the block boundary (i.e. tkv is not a multiple 
     of 64 at the time of prefilling).
@@ -191,12 +199,12 @@ def test_prompts_misaligned_with_tkv_boundaries(
             * 2: len = 41, max tokens = 67, step joining = 0
             * 3: len = 47, max tokens = 9, step joining = 0
     """
-
     seqs_max_tokens = [57, 67, 9]
     prompts_lengths = [49, 41, 47]
     steps_add_reqs = [0, 0, 0]  # add all requests in the beginning
     available_blocks = -1  # no restriction
     max_num_seqs = 2
+    max_model_len = 256
 
     checked_steps = [
         {
@@ -256,15 +264,15 @@ def test_prompts_misaligned_with_tkv_boundaries(
         },
         {
             # Prefill sequence 2
-            # total blocks in use: 4 - 2 + 2 = 4
+            # total blocks in use: 4 - 2 + 1 = 3
             "step": 59,
             "tkv": 120,  # Tkv doesn't increase because it is a prefill
             "waiting": [],
             "running": ["2", "1"],
             "request_outputs": ["2"],
-            # 5 - 2 (seq 0) + 2 (prefill (2 block) + 8 decodes in 2nd block)
-            "n_reserved_blocks": 5,
-            "n_used_blocks": 4
+            # 5 - 2 (seq 0) + 1 (prefill (1 block) + 8 decodes in 1st block)
+            "n_reserved_blocks": 4,
+            "n_used_blocks": 3
         },
         {
             # Decode sequences 1 and 2
@@ -273,8 +281,8 @@ def test_prompts_misaligned_with_tkv_boundaries(
             "waiting": [],
             "running": ["2", "1"],
             "request_outputs": ["2", "1"],
-            "n_reserved_blocks": 5,
-            "n_used_blocks": 4
+            "n_reserved_blocks": 4,
+            "n_used_blocks": 3
         },
         {
             # Sequence 2 finishes at step 68
@@ -285,18 +293,18 @@ def test_prompts_misaligned_with_tkv_boundaries(
             "running": ["1"],
             "request_outputs": ["2", "1"],
             "finished_requests": ["2"],
-            "n_reserved_blocks": 5,
-            "n_used_blocks": 4
+            "n_reserved_blocks": 4,
+            "n_used_blocks": 3
         },
         {
             # Decode sequences 1
-            # total blocks in use: 4 - 2 + 1 = 3
+            # total blocks in use: 3 - 1 + 1 = 3
             "step": 68,
             "tkv": 129,
             "waiting": [],
             "running": ["1"],
             "request_outputs": ["1"],
-            "n_reserved_blocks": 3,  # 5 - 2 (seq 2)
+            "n_reserved_blocks": 3,  # 4 - 1 (seq 2)
             "n_used_blocks": 3
         },
         {
@@ -323,7 +331,7 @@ def test_prompts_misaligned_with_tkv_boundaries(
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -332,16 +340,21 @@ def test_prompts_misaligned_with_tkv_boundaries(
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
 
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_two_sequences_finish_same_time_as_new_arrive(
-        model: str, backend: str, monkeypatch: pytest.MonkeyPatch):
+        model: str, backend: str, monkeypatch: pytest.MonkeyPatch,
+        set_random_seed):
     """ 2-cases-in-1: (1) Two sequences finish at the same time and (2) a new
     request arrives when another finishes.
 
@@ -352,12 +365,12 @@ def test_two_sequences_finish_same_time_as_new_arrive(
             * 2: len = 30, max tokens = 30, step joining = 0
             * 3: len = 20, max tokens = 10, step joining = 31
     """
-
     seqs_max_tokens = [30, 30, 10]
     prompts_lengths = [49, 30, 20]
     steps_add_reqs = [0, 0, 31]
     available_blocks = -1  # no restriction
     max_num_seqs = 2
+    max_model_len = 256
 
     checked_steps = [
         {
@@ -461,7 +474,7 @@ def test_two_sequences_finish_same_time_as_new_arrive(
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -470,16 +483,290 @@ def test_two_sequences_finish_same_time_as_new_arrive(
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
 
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
+
+@pytest.mark.cb
+@pytest.mark.parametrize("model", get_spyre_model_list())
+@pytest.mark.parametrize("backend", get_spyre_backend_list())
+def test_new_sequence_joins_during_decode(model: str, backend: str,
+                                          monkeypatch: pytest.MonkeyPatch,
+                                          set_random_seed):
+    """ Scenario where a new sequence joins while decoding other sequences
+
+    Configuration:
+        * max_num_seqs: 4
+        * number of prompts: 4
+            * 1: len = 49, max tokens = 119, step joining = 0
+            * 2: len = 14, max tokens = 52, step joining = 0
+            * 3: len = 89, max tokens = 104, step joining = 32
+            * 4: len = 9, max tokens = 64, step joining = 131
+    """
+    # TODO change to 65 max_tokens for last prompt if ever possible
+
+    seqs_max_tokens = [119, 52, 104, 64]
+    prompts_lengths = [49, 14, 89, 9]
+    steps_add_reqs = [0, 0, 32, 131]
+    available_blocks = -1  # no restriction
+    max_num_seqs = 4
+    max_model_len = 256
+
+    checked_steps = [
+        {
+            "step": 0,
+            "tkv": 0,
+            "waiting": ["0", "1"],
+            "running": [],
+            "request_outputs": [],
+            "n_reserved_blocks": 0,
+            "n_used_blocks": 0
+        },
+        {
+            # Prefill sequence 0
+            "step": 1,
+            "tkv": 64,
+            "waiting": ["1"],
+            "running": ["0"],
+            "request_outputs": ["0"],
+            "n_reserved_blocks": 3,  # prefill (1 block) + 119 decode (2 block)
+            "n_used_blocks": 1
+        },
+        {
+            # Prefill sequence 1
+            "step": 2,
+            "tkv": 64,
+            "waiting": [],
+            "running": ["1", "0"],
+            "request_outputs": ["1"],
+            "n_reserved_blocks": 5,  # prefill (1 block) + 51 decodes (1 block)
+            "n_used_blocks": 2
+        },
+        {
+            # Decode sequences 0 and 1
+            "step": 3,
+            "tkv": 65,
+            "waiting": [],
+            "running": ["1", "0"],
+            "request_outputs": ["1", "0"],
+            "n_reserved_blocks": 5,
+            "n_used_blocks": 4  # 2 blocks extended, one for each sequence
+        },
+        {
+            # Sequence 2 joins: one iteration in waiting queue
+            "step": 32,
+            "tkv": 94,
+            "waiting": ["2"],
+            "running": ["1", "0"],
+            "request_outputs": ["1", "0"],
+            "n_reserved_blocks": 5,
+            "n_used_blocks": 4
+        },
+        {
+            # Prefill sequence 2
+            "step": 33,
+            "tkv": 94,
+            "waiting": [],
+            "running": ["2", "1", "0"],
+            "request_outputs": ["2"],
+            "n_reserved_blocks": 9,  # prefill (2 block) + 103 decode (2 block)
+            "n_used_blocks": 6
+        },
+        {
+            # Decode sequences 0, 1, and 2
+            "step": 34,
+            "tkv": 95,
+            "waiting": [],
+            "running": ["2", "1", "0"],
+            "request_outputs": ["2", "1", "0"],
+            "n_reserved_blocks": 9,
+            "n_used_blocks": 6
+        },
+        {
+            # Sequence 1 finishes at step 54
+            # (start step + 2 prefills + 51 decodes - 1) = 2 + 2 + 51 - 1 = 54
+            "step": 54,
+            "tkv": 115,
+            "waiting": [],
+            "running": ["2", "0"],
+            "request_outputs": ["2", "1", "0"],
+            "finished_requests": ["1"],
+            "n_reserved_blocks": 9,
+            "n_used_blocks": 6
+        },
+        {
+            # Decode sequences 0 and 2
+            "step": 55,
+            "tkv": 116,
+            "waiting": [],
+            "running": ["2", "0"],
+            "request_outputs": ["2", "0"],
+            "n_reserved_blocks": 7,  # two blocks released
+            "n_used_blocks": 4  # two blocks released
+        },
+        {
+            # Decode sequences 0 and 2, tkv arrives to new block
+            "step": 68,
+            "tkv": 129,
+            "waiting": [],
+            "running": ["2", "0"],
+            "request_outputs": ["2", "0"],
+            "n_reserved_blocks": 7,
+            "n_used_blocks": 6  # 2 blocks extended, one for each sequence
+        },
+        {
+            # Sequence 0 finishes at step 121
+            # (start step + 3 prefills + 118 decode - 1) = 1 + 3 + 118 - 1 = 121
+            "step": 121,
+            "tkv": 182,
+            "waiting": [],
+            "running": ["2"],
+            "request_outputs": ["2", "0"],
+            "finished_requests": ["0"],
+            "n_reserved_blocks": 7,
+            "n_used_blocks": 6
+        },
+        {
+            # Decode sequence 2
+            "step": 122,
+            "tkv": 183,
+            "waiting": [],
+            "running": ["2"],
+            "request_outputs": ["2"],
+            "n_reserved_blocks": 4,  # 3 blocks released
+            "n_used_blocks": 3  # 3 blocks released
+        },
+        {
+            # Sequence 3 joins: one iteration in waiting queue
+            "step": 131,
+            "tkv": 192,
+            "waiting": ["3"],
+            "running": ["2"],
+            "request_outputs": ["2"],
+            "n_reserved_blocks": 4,
+            "n_used_blocks": 3
+        },
+        {
+            # Prefill sequence 3
+            "step": 132,
+            "tkv": 192,
+            "waiting": [],
+            "running": ["3", "2"],
+            "request_outputs": ["3"],
+            # Note: here is where the optimization happens: we do the prefill
+            # on a single block only instead of using 3 blocks
+            "n_reserved_blocks": 6,  # prefill (1 block) + 63 decode (1 block)
+            "n_used_blocks": 4  # prefill (1 block)
+        },
+        {
+            # Decode sequences 2 and 3
+            "step": 133,
+            "tkv": 193,
+            "waiting": [],
+            "running": ["3", "2"],
+            "request_outputs": ["3", "2"],
+            "n_reserved_blocks": 6,
+            "n_used_blocks": 6  # 2 blocks extended, one for each sequence
+        },
+        {
+            # Sequence 2 finishes at step 137
+            # (start step + 2 prefills + 103 decodes) = 33 + 2 + 103 - 1 = 137
+            "step": 137,
+            "tkv": 197,
+            "waiting": [],
+            "running": ["3"],
+            "request_outputs": ["3", "2"],
+            "finished_requests": ["2"],
+            "n_reserved_blocks": 6,
+            "n_used_blocks": 6
+        },
+        {
+            # Decode sequence 3
+            "step": 138,
+            "tkv": 70,
+            "waiting": [],
+            "running": ["3"],
+            "request_outputs": ["3"],
+            # 4 blocks freed due to finished sequence 2
+            "n_reserved_blocks": 2,
+            "n_used_blocks": 2
+        },
+        {
+            # Sequence 3 finishes at step 196
+            # (start step + 1 prefills + 103 decodes) = 132 + 1 + 63 - 1 = 196
+            "step": 195,
+            "tkv": 127,
+            "waiting": [],
+            "running": [],
+            "request_outputs": ["3"],
+            "finished_requests": ["3"],
+            "n_reserved_blocks": 2,
+            "n_used_blocks": 2
+        },
+        {
+            # Tkv should be cleared one step later
+            "step": 196,
+            "tkv": 0,
+            "waiting": [],
+            "running": [],
+            "request_outputs": [],
+            "n_reserved_blocks": 0,
+            "n_used_blocks": 0
+        },
+        # TODO this is when max_tokens = 65 for last prompt
+        # {
+        #     # Sequence 3 finishes at step 196
+        #     # (start step + 1 prefills + 103 decodes) = 132 + 1 + 64 - 1 = 196
+        #     "step": 196,
+        #     "tkv": 128,
+        #     "waiting": [],
+        #     "running": [],
+        #     "request_outputs": ["3"],
+        #     "finished_requests": ["3"],
+        #     "n_reserved_blocks": 2,
+        #     "n_used_blocks": 2
+        # },
+        # {
+        #     # Tkv should be cleared one step later
+        #     "step": 197,
+        #     "tkv": 0,
+        #     "waiting": [],
+        #     "running": [],
+        #     "request_outputs": [],
+        #     "n_reserved_blocks": 0,
+        #     "n_used_blocks": 0
+        # },
+    ]
+
+    cb_outputs, prompts = check_scheduler_inference_steps(
+        model=model,
+        backend=backend,
+        monkeypatch=monkeypatch,
+        seqs_max_tokens=seqs_max_tokens,
+        prompts_lengths=prompts_lengths,
+        steps_add_reqs=steps_add_reqs,
+        checked_steps=checked_steps,
+        max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
+        available_blocks=available_blocks,
+        use_cb=True,
+    )
+
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_prompt_too_long_for_current_tkv(model: str, backend: str,
-                                         monkeypatch: pytest.MonkeyPatch):
+                                         monkeypatch: pytest.MonkeyPatch,
+                                         set_random_seed):
     """ Scenario where the requested prompt is too long for current tkv value
 
     Configuration:
@@ -494,6 +781,7 @@ def test_prompt_too_long_for_current_tkv(model: str, backend: str,
     steps_add_reqs = [0, 0]
     available_blocks = -1  # no restriction
     max_num_seqs = 2
+    max_model_len = 256
 
     checked_steps = [
         {
@@ -608,7 +896,7 @@ def test_prompt_too_long_for_current_tkv(model: str, backend: str,
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -617,17 +905,23 @@ def test_prompt_too_long_for_current_tkv(model: str, backend: str,
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
 
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_requested_tokens_not_fitting_remaining_space(
-        model: str, backend: str, monkeypatch: pytest.MonkeyPatch):
-    """ Scenario where the request goes beyond max_model_len 
+        model: str, backend: str, monkeypatch: pytest.MonkeyPatch,
+        set_random_seed):
+    """ Scenario where the request goes beyond max_model_len and needs to wait
+    for a new batch.
 
     Configuration:
         * max_num_seqs: 2
@@ -636,12 +930,12 @@ def test_requested_tokens_not_fitting_remaining_space(
             * 2: len = 49, max tokens = 57, step joining = 0
             * 3: len = 41, max tokens = 80, step joining = 0
     """
-
     seqs_max_tokens = [67, 57, 80]
     prompts_lengths = [70, 49, 41]
     steps_add_reqs = [0, 0, 0]
     available_blocks = -1  # no restriction
     max_num_seqs = 2
+    max_model_len = 256
 
     checked_steps = [
         {
@@ -667,26 +961,26 @@ def test_requested_tokens_not_fitting_remaining_space(
         },
         {
             # Prefill sequence 1
-            # total blocks in use: 2 + 2
+            # total blocks in use: 2 + 1
             "step": 2,
             "tkv": 128,
             "waiting": ["2"],
             "running": ["1", "0"],
             "request_outputs": ["1"],
-            # prefill (2 blocks) + 56 decodes (1 block)
-            "n_reserved_blocks": 7,
-            "n_used_blocks": 4
+            # prefill (1 block) + 56 decodes (1 block)
+            "n_reserved_blocks": 6,
+            "n_used_blocks": 3
         },
         {
             # Decode sequences 0 and 1
-            # total blocks in use: 4 + 2 (decodes)
+            # total blocks in use: 3 + 2 (decodes)
             "step": 3,
             "tkv": 129,
             "waiting": ["2"],
             "running": ["1", "0"],
             "request_outputs": ["1", "0"],
-            "n_reserved_blocks": 7,
-            "n_used_blocks": 6
+            "n_reserved_blocks": 6,
+            "n_used_blocks": 5
         },
         {
             # Sequence 1 finishes at step 58
@@ -697,19 +991,19 @@ def test_requested_tokens_not_fitting_remaining_space(
             "running": ["0"],
             "request_outputs": ["1", "0"],
             "finished_requests": ["1"],
-            "n_reserved_blocks": 7,
-            "n_used_blocks": 6
+            "n_reserved_blocks": 6,
+            "n_used_blocks": 5
         },
         {
             # Decode sequence 0
             # Cannot prefill sequence 2: 185 + 80 = 265 > 256
-            # total blocks in use: 6 - 3 = 3
+            # total blocks in use: 5 - 2 = 3
             "step": 59,
             "tkv": 185,
             "waiting": ["2"],
             "running": ["0"],
             "request_outputs": ["0"],
-            "n_reserved_blocks": 4,  # 7 - 3 (seq 1)
+            "n_reserved_blocks": 4,  # 6 - 2 (seq 1)
             "n_used_blocks": 3
         },
         {
@@ -793,7 +1087,7 @@ def test_requested_tokens_not_fitting_remaining_space(
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -802,16 +1096,21 @@ def test_requested_tokens_not_fitting_remaining_space(
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
 
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_requests_use_all_available_blocks(model: str, backend: str,
-                                           monkeypatch: pytest.MonkeyPatch):
+                                           monkeypatch: pytest.MonkeyPatch,
+                                           set_random_seed):
     """ Scenario where the requests use all of the available blocks 
     
     Configuration:
@@ -823,13 +1122,14 @@ def test_requests_use_all_available_blocks(model: str, backend: str,
             * 4: len = 10, max tokens = 3, step joining = 0
         * available_blocks: 8
     """
-
     seqs_max_tokens = [3, 3, 3, 3]  # 2 decodes into a new block per sequence
     prompts_lengths = [10, 10, 10, 10]  # 1 block for prefil per sequence
     steps_add_reqs = [0, 0, 0, 0]
     # total number of blocks needed if scheduled together : 4 * (1 + 1) = 8
     available_blocks = 8
     max_num_seqs = 4
+    max_model_len = 256
+
     checked_steps = [
         {
             "step": 0,
@@ -924,7 +1224,7 @@ def test_requests_use_all_available_blocks(model: str, backend: str,
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -933,16 +1233,21 @@ def test_requests_use_all_available_blocks(model: str, backend: str,
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
 
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
+
 
 @pytest.mark.cb
 @pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize("backend", get_spyre_backend_list())
 def test_requests_use_more_than_available_blocks(
-        model: str, backend: str, monkeypatch: pytest.MonkeyPatch):
+        model: str, backend: str, monkeypatch: pytest.MonkeyPatch,
+        set_random_seed):
     """ Scenario where some request need to wait because of the number of 
     available blocks. 
     
@@ -962,6 +1267,8 @@ def test_requests_use_more_than_available_blocks(
     # total number of blocks needed if scheduled together : 4 * (1 + 1) = 8
     available_blocks = 4
     max_num_seqs = 4
+    max_model_len = 256
+
     checked_steps = [
         {
             "step": 0,
@@ -1081,7 +1388,7 @@ def test_requests_use_more_than_available_blocks(
         },
     ]
 
-    check_scheduler_inference_steps(
+    cb_outputs, prompts = check_scheduler_inference_steps(
         model=model,
         backend=backend,
         monkeypatch=monkeypatch,
@@ -1090,6 +1397,10 @@ def test_requests_use_more_than_available_blocks(
         steps_add_reqs=steps_add_reqs,
         checked_steps=checked_steps,
         max_num_seqs=max_num_seqs,
+        max_model_len=max_model_len,
         available_blocks=available_blocks,
         use_cb=True,
     )
+
+    check_output_against_hf(model, backend, seqs_max_tokens, cb_outputs,
+                            prompts)
