@@ -4,9 +4,9 @@ Run `python -m pytest tests/e2e/test_spyre_warmup_shapes.py`.
 """
 
 import pytest
-from spyre_util import (compare_results, generate_hf_output,
-                        generate_spyre_vllm_output, get_chicken_soup_prompts,
-                        get_spyre_backend_list, get_spyre_model_list)
+from spyre_util import (check_output_against_hf, generate_spyre_vllm_output,
+                        get_chicken_soup_prompts, get_spyre_backend_list,
+                        get_spyre_model_list)
 from vllm import SamplingParams
 
 
@@ -57,16 +57,8 @@ def test_output(
         backend=backend,
         monkeypatch=monkeypatch)
 
-    hf_results = generate_hf_output(model=model,
-                                    prompts=prompts,
-                                    max_new_tokens=max_new_tokens)
-
-    compare_results(model=model,
-                    prompts=prompts,
-                    tensor_parallel_size=1,
-                    backend=backend,
-                    vllm_results=vllm_results,
-                    hf_results=hf_results)
+    check_output_against_hf(model, backend, max_new_tokens, vllm_results,
+                            prompts)
 
 
 @pytest.mark.parametrize("model", get_spyre_model_list())
