@@ -1,5 +1,6 @@
 import math
 import os
+import random
 import subprocess
 import sys
 import time
@@ -10,6 +11,7 @@ import numpy as np
 import openai
 import pytest
 import requests
+import torch
 from sentence_transformers import SentenceTransformer, util
 from transformers import AutoModelForCausalLM, AutoTokenizer
 from vllm import LLM, SamplingParams
@@ -152,24 +154,27 @@ class RemoteOpenAIServer:
                                   **kwargs)
 
 
-def patch_warmup_shapes(warmup_shapes: list[tuple[int, int, int]],
+def patch_warmup_shapes(warmup_shapes: Union[list[tuple[int, int, int]],
+                                             list[tuple[int, int]]],
                         monkeypatch):
     warmup_prompt_length = [t[0] for t in warmup_shapes]
-    warmup_new_tokens = [t[1] for t in warmup_shapes]
-    warmup_batch_size = [t[2] for t in warmup_shapes]
+    warmup_batch_size = [t[-1] for t in warmup_shapes]
 
     monkeypatch.setenv('VLLM_SPYRE_WARMUP_PROMPT_LENS',
                        ','.join(str(val) for val in warmup_prompt_length))
-    monkeypatch.setenv('VLLM_SPYRE_WARMUP_NEW_TOKENS',
-                       ','.join(str(val) for val in warmup_new_tokens))
     monkeypatch.setenv('VLLM_SPYRE_WARMUP_BATCH_SIZES',
                        ','.join(str(val) for val in warmup_batch_size))
 
+    if all(len(s) == 3 for s in warmup_shapes):
+        warmup_new_tokens = [t[1] for t in warmup_shapes]
+        monkeypatch.setenv('VLLM_SPYRE_WARMUP_NEW_TOKENS',
+                           ','.join(str(val) for val in warmup_new_tokens))
+
 
 # vLLM / Spyre
 def generate_spyre_vllm_output(
     model: str,
-    prompts: list[str],
+    prompts: Union[list[str], list[list[int]]],
     max_model_len: int,
     block_size: int,
     sampling_params: Union[SamplingParams, list[SamplingParams]],
@@ -198,7 +203,6 @@ def generate_spyre_vllm_output(
                            ",".join(str(val) for val in warmup_batch_size))
     # --------------
     monkeypatch.setenv("VLLM_SPYRE_USE_CB", "1" if use_cb else "0")
-    monkeypatch.setenv("VLLM_USE_V1", "1")
     monkeypatch.setenv("VLLM_SPYRE_DYNAMO_BACKEND", backend)
 
     # Allows to run multiprocess V1 engine without dumping meaningless logs at
@@ -240,18 +244,25 @@ def generate_spyre_vllm_output(
 
 # Hugging Face
 def generate_hf_output(
-        model: str, prompts: list[str],
-        max_new_tokens: Union[int, list[int]]) -> list[dict[str, Any]]:
+    model: str,
+    prompts: Union[list[str], list[list[int]]],  # also accept token ids
+    max_new_tokens: Union[int, list[int]],
+    ignore_eos: bool = False,
+) -> list[dict[str, Any]]:
 
     if not isinstance(max_new_tokens, list):
         max_new_tokens = [max_new_tokens] * len(prompts)
 
     hf_model = AutoModelForCausalLM.from_pretrained(model)
     hf_tokenizer = AutoTokenizer.from_pretrained(model)
+    if ignore_eos:
+        hf_model.generation_config.eos_token_id = None
 
     results = []
     for prompt_index, prompt in enumerate(prompts):
-        hf_input_tokens = hf_tokenizer(prompt, return_tensors="pt").input_ids
+        hf_input_tokens = hf_tokenizer(prompt, return_tensors="pt").input_ids \
+                    if isinstance(prompt[0], str) \
+                    else torch.tensor([prompts[prompt_index]])
         hf_output = hf_model.generate(
             hf_input_tokens,
             do_sample=False,
@@ -286,9 +297,16 @@ def generate_hf_output(
 
 
 # compare results
-def compare_results(model: str, prompts: list[str], tensor_parallel_size: int,
-                    backend: str, vllm_results: list[dict[str, Any]],
-                    hf_results: list[dict[str, Any]]):
+def compare_results(
+    model: str,
+    tensor_parallel_size: int,
+    backend: str,
+    vllm_results: list[dict[str, Any]],
+    hf_results: list[dict[str, Any]],
+    prompts: Optional[list[str]] = None,
+):
+    if prompts is None:
+        prompts = [""] * len(vllm_results)
 
     print(f"\nmodel:         {model:s}")
     print(f"tp size:       {tensor_parallel_size}")
@@ -305,12 +323,14 @@ def compare_results(model: str, prompts: list[str], tensor_parallel_size: int,
 
     for prompt_index, (prompt, hf_result, vllm_result) in enumerate(
             zip(prompts, hf_results, vllm_results)):
-        err_msg = '' if hf_result['text'] == vllm_result['text'] else '  ERROR'
-        print(f"\nprompt {prompt_index:3d}:    {repr(prompt):s}")
-        print("generated:")
-        print(f"        HF:    {repr(hf_result['text']):s}")
-        print(f"        vLLM:  {repr(vllm_result['text']):s}{err_msg}")
-        print()
+        if "text" in vllm_result:
+            err_msg = '' if hf_result['text'] == vllm_result[
+                'text'] else '  ERROR'
+            print(f"\nprompt {prompt_index:3d}:    {repr(prompt):s}")
+            print("generated:")
+            print(f"        HF:    {repr(hf_result['text']):s}")
+            print(f"        vLLM:  {repr(vllm_result['text']):s}{err_msg}")
+            print()
 
         assert DISABLE_ASSERTS or backend == 'sendnn' or\
             hf_result['token_ids'] == vllm_result['token_ids']
@@ -322,20 +342,23 @@ def compare_results(model: str, prompts: list[str], tensor_parallel_size: int,
             logprob_abs_diff_list = []
             logprob_rel_diff_list = []
 
-            for hf_token, hf_token_id, hf_logprob, vllm_token,\
-                 vllm_token_id, vllm_logprob in zip(
-                    hf_result['tokens'], hf_result['token_ids'],
-                    hf_result['logprobs'], vllm_result['tokens'],
-                    vllm_result['token_ids'], vllm_result['logprobs']):
+            for i, (hf_token_id, hf_logprob, vllm_token_id,
+                    vllm_logprob) in enumerate(
+                        zip(hf_result['token_ids'], hf_result['logprobs'],
+                            vllm_result['token_ids'],
+                            vllm_result['logprobs'])):
                 logprob_abs_diff = math.fabs(hf_logprob - vllm_logprob)
                 logprob_abs_diff_list.append(logprob_abs_diff)
                 logprob_rel_diff = math.fabs(logprob_abs_diff / hf_logprob)
                 logprob_rel_diff_list.append(logprob_rel_diff)
 
+                hf_token = repr(
+                    hf_result['tokens'][i]) if 'tokens' in vllm_result else '-'
+                vllm_token = repr(vllm_result['tokens']
+                                  [i]) if 'tokens' in vllm_result else '-'
                 print(
-                    f"HF: {hf_token_id:8d} {repr(hf_token):14s} "
-                    f"{hf_logprob:14f}  "
-                    f"vLLM: {vllm_token_id:8d} {repr(vllm_token):14s} "
+                    f"HF: {hf_token_id:8d} {hf_token:14s} {hf_logprob:14f}  "
+                    f"vLLM: {vllm_token_id:8d} {vllm_token:14s} "
                     f"{vllm_logprob:14f}  ",
                     end='')
 
@@ -373,25 +396,27 @@ def compare_results(model: str, prompts: list[str], tensor_parallel_size: int,
         print()
 
 
+def check_output_against_hf(model, backend, max_new_tokens, vllm_results,
+                            prompts) -> None:
+    hf_outputs = generate_hf_output(
+        model=model,
+        prompts=prompts,
+        max_new_tokens=max_new_tokens,
+        ignore_eos=True,
+    )
+    compare_results(
+        model=model,
+        tensor_parallel_size=1,
+        backend=backend,
+        vllm_results=vllm_results,
+        hf_results=hf_outputs,
+    )
+
+
 # vLLM / Spyre
-def spyre_vllm_embeddings(model: str, prompts: list[str],
-                          warmup_shapes: list[tuple[int, int]],
-                          max_model_len: int, block_size: int,
-                          tensor_parallel_size: int, backend: str,
-                          vllm_version: str) -> list[dict[str, Any]]:
-
-    warmup_prompt_length = [t[0] for t in warmup_shapes]
-    warmup_new_tokens = [0] * len(warmup_shapes)
-    warmup_batch_size = [t[1] for t in warmup_shapes]
-
-    os.environ['VLLM_SPYRE_WARMUP_PROMPT_LENS'] = ','.join(
-        str(val) for val in warmup_prompt_length)
-    os.environ['VLLM_SPYRE_WARMUP_NEW_TOKENS'] = ','.join(
-        str(val) for val in warmup_new_tokens)
-    os.environ['VLLM_SPYRE_WARMUP_BATCH_SIZES'] = ','.join(
-        str(val) for val in warmup_batch_size)
-    os.environ['VLLM_SPYRE_DYNAMO_BACKEND'] = backend
-    os.environ['VLLM_USE_V1'] = "1" if vllm_version == "V1" else "0"
+def spyre_vllm_embeddings(model: str, prompts: list[str], max_model_len: int,
+                          block_size: int, tensor_parallel_size: int,
+                          backend: str) -> list[dict[str, Any]]:
 
     vllm_model = LLM(model=model,
                      tokenizer=model,
@@ -539,22 +564,46 @@ def create_text_prompt(model: str, min_tokens: int, max_tokens: int) -> str:
     return prompt
 
 
-def create_random_request(
-        request_id: int, num_tokens: int,
-        sampling_params: SamplingParams) -> EngineCoreRequest:
+def create_random_request(request_id: int,
+                          num_tokens: int,
+                          sampling_params: SamplingParams,
+                          from_model_vocab: bool = False,
+                          model: Optional[str] = None) -> EngineCoreRequest:
 
-    return EngineCoreRequest(request_id=str(request_id),
-                             prompt_token_ids=[request_id] * num_tokens,
-                             mm_inputs=None,
-                             mm_hashes=None,
-                             mm_placeholders=None,
-                             sampling_params=sampling_params,
-                             eos_token_id=None,
-                             arrival_time=0,
-                             lora_request=None,
-                             data_parallel_rank=None,
-                             pooling_params=None,
-                             cache_salt=None)
+    tokenizer = AutoTokenizer.from_pretrained(model)
+    if from_model_vocab:
+        assert model is not None, "Prompt requested to be generated from " \
+        "model's vocabulary: need to provide model."
+
+        valid_token_ids = sorted([
+            v for v in tokenizer.vocab.values()
+            if v not in tokenizer.all_special_ids
+        ])
+        prompt_token_ids = random.choices(valid_token_ids, k=num_tokens)
+    else:
+        # start with existing prompts and tokenize them
+        prompts = get_longer_chicken_soup_prompts(1)
+        tokenized_prompts = tokenizer(prompts)["input_ids"]
+        prompt_token_ids = [p[:num_tokens] for p in tokenized_prompts][0]
+
+        # make sure we get enough tokens from the prompts
+        assert (len(prompt_token_ids) == num_tokens
+                ), f"need {num_tokens} but got {len(prompt_token_ids)}"
+
+    return EngineCoreRequest(
+        request_id=str(request_id),
+        prompt_token_ids=prompt_token_ids,
+        mm_inputs=None,
+        mm_hashes=None,
+        mm_placeholders=None,
+        sampling_params=sampling_params,
+        eos_token_id=None,
+        arrival_time=0,
+        lora_request=None,
+        data_parallel_rank=None,
+        pooling_params=None,
+        cache_salt=None,
+    )
 
 
 def skip_unsupported_tp_size(size: int, backend: str):
@@ -591,3 +640,40 @@ def get_chicken_soup_prompts(num_prompts: int) -> list[str]:
         prompts = prompts * (math.ceil(num_prompts / 4))
 
     return prompts[:num_prompts]
+
+
+def get_longer_chicken_soup_prompts(num_prompts: int) -> list[str]:
+    template = (
+        "Below is an instruction that describes a task. Write a response that "
+        "appropriately completes the request. Be polite in your response to the"
+        " user.\n\n### Instruction:\n{}\n\n### Response:")
+
+    prompts = [
+        template.format("Provide a list of instructions "
+                        "for preparing chicken soup along with "
+                        "rice curry to go with it so that "
+                        "the flavor is amazing and make sure to follow the "
+                        "recipe that my mum used to make during my "
+                        "childhood so that I can relive my good "
+                        "memories thanks"),
+        template.format("Provide me a list of things that I can do with my "
+                        "new found wealth which I have obtained through "
+                        "nefarious activities including gambling "
+                        "and betting on sports thanks"),
+        template.format(
+            "how do I add multiple new columns in m for power query or \
+                power bi? Can you explain that to me like I'm 5 years old "
+            "with thorough step by step explanation and covering all edge "
+            "cases thanks"),
+        template.format(
+            "Convert char to string in Java "
+            "and write unit tests for the same, making sure they all pass "
+            "and we get amazing test coverage along with high level "
+            "correctness so that the PR reviewers have an easy time "
+            "reviewing the changes thanks"),
+    ]
+
+    if num_prompts > 4:
+        prompts = prompts * (math.ceil(num_prompts / 4))
+
+    return prompts[:num_prompts]
