@@ -9,7 +9,6 @@ if TYPE_CHECKING:
     VLLM_SPYRE_WARMUP_NEW_TOKENS: Optional[list[int]] = None
     VLLM_SPYRE_WARMUP_BATCH_SIZES: Optional[list[int]] = None
     VLLM_SPYRE_USE_CB: bool = False
-    VLLM_SPYRE_N_BLOCKS: int = 0
     VLLM_SPYRE_PERF_METRIC_LOGGING_ENABLED: int = 0
     VLLM_SPYRE_PERF_METRIC_LOGGING_DIR: str = "/tmp"
     VLLM_SPYRE_OVERRIDE_SIGNALS_HANDLER: bool = False
@@ -19,6 +18,7 @@ if TYPE_CHECKING:
     # many cases, so it should only be enabled when prompt_logprobs are required
     # for experimentation purposes.
     VLLM_SPYRE_ENABLE_PROMPT_LOGPROBS: bool = False
+    VLLM_SPYRE_UPDATE_THREAD_CONFIG: bool = True
 
 logger = init_logger(__name__)
 
@@ -75,10 +75,6 @@ environment_variables: dict[str, Callable[[], Any]] = {
     "VLLM_SPYRE_USE_CB":
     lambda: bool(int(os.getenv("VLLM_SPYRE_USE_CB", "0"))),
 
-    # Overriding the number of KV cache blocks available on Spyre (and CPU)
-    "VLLM_SPYRE_N_BLOCKS":
-    lambda: int(os.getenv("VLLM_SPYRE_N_BLOCKS", 0)),
-
     # Enable performance metric logging. This captures startup information
     # such as warmup times, and loading times. It is turned off by default.
     "VLLM_SPYRE_PERF_METRIC_LOGGING_ENABLED":
@@ -99,6 +95,11 @@ environment_variables: dict[str, Callable[[], Any]] = {
     # By default, prompt_logprobs aren't supported
     "VLLM_SPYRE_ENABLE_PROMPT_LOGPROBS":
     lambda: bool(int(os.getenv("VLLM_SPYRE_ENABLE_PROMPT_LOGPROBS", "0"))),
+
+    # Allow vllm-spyre to update env vars related to multi-threading (eg. OMP)
+    # based on the detected CPU cores and server configuration
+    "VLLM_SPYRE_UPDATE_THREAD_CONFIG":
+    lambda: bool(int(os.getenv("VLLM_SPYRE_UPDATE_THREAD_CONFIG", "1"))),
 }
 # --8<-- [end:env-vars-definition]
 
