@@ -16,6 +16,7 @@ from vllm.distributed import (ensure_model_parallel_initialized,
                               init_distributed_environment)
 from vllm.logger import init_logger
 from vllm.model_executor import set_random_seed
+from vllm.pooling_params import PoolingParams
 from vllm.sampling_params import SamplingParams
 from vllm.v1.core.sched.output import (CachedRequestData, NewRequestData,
                                        SchedulerOutput)
@@ -29,7 +30,8 @@ import vllm_spyre.perf_metrics as perf_metrics
 from vllm_spyre.model_executor.model_loader import spyre_setup
 from vllm_spyre.platform import SpyrePlatform
 from vllm_spyre.v1.worker.spyre_model_runner import (
-    ContinuousBatchingSpyreModelRunner, StaticBatchingSpyreModelRunner)
+    ContinuousBatchingSpyreModelRunner, SpyrePoolingModelRunner,
+    StaticBatchingSpyreModelRunner, SupportedTask)
 
 logger = init_logger(__name__)
 
@@ -54,6 +56,29 @@ class SpyreWorker(WorkerBaseV1):
     """A worker class that executes the model on a group of Spyre cores.
     """
 
+    def get_supported_pooling_tasks(self):
+        # Compatibility code required for vllm == 0.10.0
+        # Can be removed for vllm > 0.10.0
+        if self.is_pooling:
+            return ["embed"]
+        return []
+
+    @property
+    def is_pooling(self) -> bool:
+        # Can be simplified after the deprecation of `model_config.task` in
+        # vllm > 0.10.0
+        return "embed" in self.model_config.supported_tasks if (
+            self.model_config.task == "auto" or self.model_config.task
+            is None) else self.model_config.task == "embed"
+
+    @property
+    def is_decoder(self) -> bool:
+        # Can be simplified after the deprecation of `model_config.task` in
+        # vllm > 0.10.0
+        return "generate" in self.model_config.supported_tasks if (
+            self.model_config.task == "auto" or self.model_config.task
+            is None) else self.model_config.task == "generate"
+
     def get_kv_cache_spec(self) -> KVCacheSpec:
         """Get specifications for KV cache implementation.
 
@@ -82,12 +107,12 @@ class SpyreWorker(WorkerBaseV1):
             (s["prompt_length"], s["new_tokens"], s["batch_size"])
                 for s in self.spyre_warmup_shapes
         ]):
-            if self.model_config.task != "embed":
+            if not self.is_pooling:
                 # TODO: remove if spyre supports
                 # lower number of output tokens
-                assert num_decode_tokens >= 3, (
+                assert num_decode_tokens >= 2, (
                     "VLLM_SPYRE_WARMUP_NEW_TOKENS must be "
-                    "at least 3 (spyre requirement).")
+                    "at least 2 (spyre requirement).")
             # warmup individual combination
             logger.info(
                 "[WARMUP] (%d/%d) for prompt length %d, decoding %d tokens "
@@ -95,6 +120,9 @@ class SpyreWorker(WorkerBaseV1):
                 prompt_len, num_decode_tokens, batch_size)
             self._warmup_spyre_fixed_size(prompt_len, num_decode_tokens,
                                           self.restricted_tokens, batch_size)
+
+        self.model_runner.complete_warmup()
+
         all_warmup_end_t = time.time()
         all_warmup_total_t = all_warmup_end_t - all_warmup_start_t
         self.perf_metrics.log("total warmup time", all_warmup_total_t)
@@ -104,7 +132,6 @@ class SpyreWorker(WorkerBaseV1):
             "[WARMUP] All %d prompt/decode/batchsize-shape "
             "combinations finished in %.3fs", num_shape_combinations,
             all_warmup_total_t)
-        self.model_runner.complete_warmup()
 
     def check_health(self) -> None:
         """Basic health check (override for device-specific checks)."""
@@ -164,9 +191,12 @@ class SpyreWorker(WorkerBaseV1):
             init_cached_hf_modules()
         self.model_runner: \
             Union[StaticBatchingSpyreModelRunner,
-                  ContinuousBatchingSpyreModelRunner]
-        if self.model_config.task == "embed":
-            raise NotImplementedError
+                  ContinuousBatchingSpyreModelRunner, SpyrePoolingModelRunner]
+        if self.is_pooling:
+            self.model_runner = SpyrePoolingModelRunner(
+                self.vllm_config, self.is_driver_worker)
+            self.spyre_warmup_shapes = SpyrePlatform.get_warmup_shapes(
+                self.vllm_config.scheduler_config)
         else:
             if envs_spyre.VLLM_SPYRE_USE_CB:
                 self.model_runner = ContinuousBatchingSpyreModelRunner(
@@ -333,10 +363,10 @@ class SpyreWorker(WorkerBaseV1):
                 mm_hashes=[],
                 mm_positions=[],
                 sampling_params=SamplingParams(max_tokens=num_decode_tokens),
+                pooling_params=None,
                 block_ids=[0],  # not actually used
                 num_computed_tokens=0,
                 lora_request=None,
-                pooling_params=None,
             ) for i in range(batch_size + 1)
         ]
         add_dummy_request = dummy_requests.pop(-1)
@@ -368,7 +398,7 @@ class SpyreWorker(WorkerBaseV1):
         self.execute_model(scheduler_output)
         self._cleanup_model_runner(request=[add_dummy_request])
 
-        model_runner.finish_warmup()
+        model_runner.complete_warmup()
 
         warmup_end_t = time.time()
         warmup_total_t = warmup_end_t - warmup_start_t
@@ -425,19 +455,24 @@ class SpyreWorker(WorkerBaseV1):
         warmup_tokens_tensor = valid_token_ids_tensor[torch.randint(
             0, len(valid_token_ids_tensor), (batch_size, prompt_len))]
 
+        sampling_params, pooling_params = None, None
+        if not self.is_pooling:
+            sampling_params = SamplingParams(max_tokens=num_decode_tokens)
+        else:
+            pooling_params = PoolingParams()
+
         # Set up dummy requests for prefill steps
         dummy_requests = [
-            NewRequestData(
-                req_id="warmup",
-                prompt_token_ids=warmup_tokens_tensor[i].tolist(),
-                mm_inputs=[],
-                mm_hashes=[],
-                mm_positions=[],
-                sampling_params=SamplingParams(max_tokens=num_decode_tokens),
-                block_ids=[0],
-                num_computed_tokens=0,
-                lora_request=None,
-                pooling_params=None) for i in range(batch_size)
+            NewRequestData(req_id="warmup",
+                           prompt_token_ids=warmup_tokens_tensor[i].tolist(),
+                           mm_inputs=[],
+                           mm_hashes=[],
+                           mm_positions=[],
+                           sampling_params=sampling_params,
+                           pooling_params=pooling_params,
+                           block_ids=[0],
+                           num_computed_tokens=0,
+                           lora_request=None) for i in range(batch_size)
         ]
 
         # Set up dummy cached_requests for decode steps
@@ -616,6 +651,9 @@ class SpyreWorker(WorkerBaseV1):
     def kv_cache(self) -> Optional[list[list[torch.Tensor]]]:
         return None
 
+    def get_supported_tasks(self) -> tuple[SupportedTask, ...]:
+        return self.model_runner.get_supported_tasks()
+
     @SpyrePlatform.inference_mode()
     def execute_model(
         self,
@@ -633,7 +671,7 @@ class SpyreWorker(WorkerBaseV1):
 # handler from vLLM when it starts a process for the engine code. Therefore,
 # the engine does not have a chance to gracefully shutdown.
 def maybe_override_signals_handler():
-    if not (envs.VLLM_USE_V1 and envs.VLLM_ENABLE_V1_MULTIPROCESSING
+    if not (envs.VLLM_ENABLE_V1_MULTIPROCESSING
             and envs_spyre.VLLM_SPYRE_OVERRIDE_SIGNALS_HANDLER):
         return
 
